/*
  HashFile: una libreria per leggere e scrivere file di stringhe a lunghezza
  fissa gestite attraverso una tabella hash.
  Versione originale C++: 10/12/2003
  Porting ANSI C: 30/10/2007
  Autore: Silvio Moioli
*/

#include "FileIO.h"
#include "HashFile.h"

/* Crea un nuovo file di hash */
HashFile* newHashFile(){
    HashFile* result = malloc(sizeof(HashFile));
    result->fp = newFile();
	result->n = 0;
	result->collisioni = 0;
	return result;
}

/* Aggiunge un record, torna 0 se fallisce */
int hashFileAdd(HashFile* hf, char* s){
	char request[RECORDLENGTH] = {0};
	char result[RECORDLENGTH] = {0};
	int hash = 0;
	if (hf->n >= RECORDSINFILE){
		return 0;/* il file e' pieno... */
	}
	strncpy(request, s, RECORDLENGTH);

	hash = hashFileWhere(hf, request);
	printf("Hash: %d\n", hash);
	fileWrite(hf->fp, s, hash);
	hf->n++;
	return 1;
}

/* Ritorna la posizione del record nel file -1 se fallisce */
int hashFileWhere(HashFile* hf, char* s){
	char request[RECORDLENGTH] = {0};
	char result[RECORDLENGTH] = {0};
	int hash = 0;
	int tentativi = 0;
	strncpy(request, s, RECORDLENGTH);

	hash = hashString(hf, request) % RECORDSINFILE;
	while(strncmp(fileRead(hf->fp, hash, result), request, RECORDLENGTH) != 0){
	    /* collisione */
	    hf->collisioni++;
		hash = (hash + 1) % RECORDSINFILE;
	    tentativi++;
		if (tentativi >= RECORDSINFILE){
			return -1;
		}
	}
	return hash;
}

/* Ritorna il numero di collisioni avvenute */
int getCollisions(HashFile* hf){
    return hf->collisioni;
}

/* Ritorna il numero di inserimenti avvenuti */
int getInsertions(HashFile* hf){
    return hf->n;
}

/* Calcola l'hash di una stringa */
int hashString(HashFile* hf, char* s){
	int i,
	result = 0,
	len = strlen(s),
	cycles = (len > RECORDLENGTH)? RECORDLENGTH : len;
	for(i = 0; i<cycles; i++){
		result^=s[i];
	}
	return result;
}